/*
 * (c) 2022, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#ifndef __EXAR_X17_D154_H__
#define __EXAR_X17_D154_H__


#include <hw/pci.h>


#define EXAR_VID                            0x13a8
#define EXAR_XR17D154_DID                   0x354
#define EXAR_XR17D158_DID                   0x358

#define ADVANTECH_VID                       0x13fe
#define ADVANTECH_PCI1612_DID_B             0xb
#define ADVANTECH_PCI1612_DID_1B            0x1b

#define EXAR_FIFO_RX_SIZE                   192
#define EXAR_FIFO_TX_SIZE                   128

#define EXAR_UART_CHANNEL_SIZE              0x400
#define EXAR_UART_CHANNEL_OFFSET            0
#define EXAR_UART_CHANNEL_BASE( uart_num )  ((uart_num) * 0x400)
#define EXAR_CONFIG_REG_BASE                0x80
#define UART_READ_FIFO_BASE( uart_num )     (EXAR_UART_CHANNEL_BASE( uart_num ) + 0x100)
#define UART_WRITE_FIFO_BASE( uart_num )    (UART_READ_FIFO_BASE( uart_num ))
#define UART_ERROR_FIFO_BASE( uart_num )    (EXAR_UART_CHANNEL_BASE( uart_num ) + 0x200)

#define UART_CONFIG_REG_INT_OFFSET          0x0
#define UART_CONFIG_REG_INT_GET_CH( regval, chnum ) (((regval) >> (chnum * 3)) & 0x7)
#define UART_DEFAULT_CH_COUNT               4
#define UART_MAX_CH_COUNT                   8
#define UART_INT_NONE                       0
#define UART_INT_RXRDY                      1
#define UART_INT_RXRDY_TO                   2
#define UART_INT_RXRDY_THR                  3
#define UART_INT_RXRDY_MSR                  4
#define UART_INT_MPIO_PINS                  6
#define UART_INT_TIMER_TO                   7
#define UART_CONFIG_REG_TIMERCTL_OFFSET     0x4
#define UART_TIMER_EN                       0x1
#define UART_TIMER_STOP_START               (0x1 << 1)
#define UART_TIMER_RETRIGGER_EN             (0x1 << 2)
#define UART_TIMER_EXT_SRC                  (0x1 << 3)
#define UART_TIMER_INT_TO_MPIO              (0x1 << 4)
#define UART_TIMER_GET_CNTR( regval )       ((val) >> 16)
#define UART_TIMER_SET_CNTR( regval, val )  (((regval) & ~(0xFFFF << 16)) | ((val) << 16))
#define UART_CONFIG_REG_OPTIONS_OFFSET      0x8
#define UART_8X_ENABLE( chnum )             ((1 << chnum) << 16)
#define UART_RESET( chnum )                 ((1 << (chnum)) << 16)
#define UART_SLEEP( chnum )                 ((1 << (chnum)) << 24)
#define UART_CONFIG_REG_ID_MPIO_INT_OFFSET  0xC
#define UART_GET_REVISION( regval )         (regval & 0xFF)
#define UART_GET_DVID( regval )             (((regval) >> 8) & 0xFF)
#define UART_GET_REGB( regval )             (((regval) >> 16) & 0xFFFF)
#define UART_MPIO_INT( mpionum )            ((1 << (mpionum)) << 24)
#define UART_CONFIG_REG_MPIO_OFFSET         0x10
#define UART_MPIO_LVL( mpionum )            (1 << mpionum)
#define UART_MPIO3T( mpionum )              ((1 << mpionum) << 8)
#define UART_MPIOINV( mpionum )             ((1 << mpionum) << 16)
#define UART_MPIOSEL( mpionum )             ((1 << mpionum) << 24)

#define UART_DEV_CLASS                      (0x70002)
#define EXAR_DEV_CLASS                      (0x70200)


typedef enum {
    EXAR_MODE_RS232,
    EXAR_MODE_RS485
} exar_mode;


typedef struct {
    void                *pci_dev_handle;
    struct pci_dev_info *pci_dev;
    uint32_t            caps;
    uint8_t             *uart_mem;
    int                 uart_cnt;
    int                 ch_offset;
    int                 disable_aspm;
    int                 rs485_invert_polarity;
    exar_mode mode[UART_MAX_CH_COUNT + 1];
} exar_dev_t;


int exar_dev_init( struct pci_dev_info *dev, exar_dev_t *exdev );
uint32_t exar_read32( exar_dev_t *exdev, uintptr_t offset );
uint8_t exar_read8( exar_dev_t *dev, uintptr_t offset );
void exar_write8( exar_dev_t *dev, uintptr_t offset, uint8_t val );
void exar_write32( exar_dev_t *exdev, uintptr_t offset, uint32_t val );
uint64_t exar_read64( exar_dev_t *exdev, uintptr_t offset );
void exar_write64( exar_dev_t *exdev, uintptr_t offset, uint64_t val );


#endif  /* __EXAR_X17_D154_H__ */
