/*
 * (c) 2022, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#ifdef DEFN
    #define EXT
    #define INIT1( a ) = { a }
#else
    #define EXT extern
    #define INIT1(a)
#endif


#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <malloc.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/neutrino.h>
#include <termios.h>
#include <devctl.h>
#include <sys/dcmd_chr.h>
#include <sys/iomsg.h>
#include <atomic.h>
#include <hw/inout.h>
#include <hw/8250.h>
#include <sys/io-char.h>
#include <sys/slog.h>
#include <sys/slogcodes.h>
#include <variant.h>
#include "exar.h"


extern exar_dev_t   exar_dev;


#ifndef DEV_8250                    /* Can be defined in variant.h to override default */
typedef struct dev_8250 {
    TTYDEV          tty;
    struct dev_8250 *next;
    unsigned        intr;
    unsigned        clk;
    unsigned        div;
    unsigned char   rx_fifo;        /* rx fifo size */
    unsigned char   tx_fifo;        /* tx fifo size */
    unsigned        dtr_disable;    /* Disable DTR control. Workaround for Qt QtSerialPort */
    uintptr_t       port[0x100];
    int             chnum;
    exar_dev_t      *parent_dev;
} DEV_8250;
#endif


struct dev_list {
    struct dev_list *next;
    DEV_8250        *device;
    int             iid;
};


EXT TTYCTRL         ttyctrl;
EXT struct dev_list *devices;


#define MAX_DEVICES                     16
#define FIFO_SIZE                       256

#include "proto.h"

#ifndef IE_SET_ALL
#define IE_SET_ALL                      (IE_RX | IE_TX | IE_LS | IE_MS)
#endif

#ifndef IE_CLR_ALL
#define IE_CLR_ALL                      0x0
#endif


extern int driver_verbosity_level;
#define DEVC_EXAR_SLOG_OPCODE           _SLOG_SETCODE( _SLOGC_CHAR, 0 )
int devc_slogf(int verbosity_level, const char *fmt, ...);
#define devc_dbg( ... )                 devc_slogf( 3, __VA_ARGS__ )
#define devc_info( ... )                devc_slogf( 2, __VA_ARGS__ )
#define devc_warn( ... )                devc_slogf( 1, __VA_ARGS__ )
#define devc_err( ... )                 devc_slogf( 0, __VA_ARGS__ )

#define REG_DLD                         2
#define REG_DLD_INVERT_RS485_POLARITY   0x80
#define REG_SPR                         7
#define REG_FCTR                        8
#define REG_FCTR_INVERT_IR_OFFSET       4
#define REG_FCTR_INVERT_IR_MASK         1
#define REG_FCTR_AUTO_RS485_OFFSET      5
#define REG_FCTR_AUTO_RS485_MASK        1
#define REG_FCTR_TRG_TABLE_OFFSET       6
#define REG_FCTR_TRG_TABLE_MASK         0x3
#define REG_FCTR_RTS_DTR_HYST_OFFSET    0
#define REG_FCTR_RTS_DTR_HYST_MASK      0xF
#define REG_EFR                         9
#define REG_EFR_ENABLE_SHARED_BITS      0x10
#define REG_TXCNT                       10
#define REG_TXTRG                       10
#define REG_RXCNT                       11
#define REG_RXTRG                       11
#define REG_XOFF1                       12
#define REG_XCHAR                       12
#define REG_XOFF2                       13
#define REG_XON1                        14
#define REG_XON2                        15

#define FIELD_VAL( var, bitname )       ((var >> bitname ## _OFFSET) & bitname ## _MASK)
#define FIELD_SET( val, fieldname )     ((val & fieldname ## _MASK) << fieldname ## _OFFSET)

#define REG_EX_INT0                     0x80
#define REG_EX_INT1                     0x81
#define REG_EX_INT2                     0x82
#define REG_EX_INT3                     0x83
#define REG_EX_TIMERCTL                 0x84
#define REG_EX_REGA                     0x85
#define REG_EX_TIMERLSB                 0x86
#define REG_EX_TIMERMSB                 0x87
#define REG_EX_8XMODE                   0x88
#define REG_EX_4XMODE                   0x89
#define REG_EX_RESET                    0x8A
#define REG_EX_SLEEP                    0x8B
#define REG_EX_DREV                     0x8C
#define REG_EX_DVID                     0x8D
#define REG_EX_REGB                     0x8E
